@@ -24,6 +24,7 @@
 # SOFTWARE.
 
 import asyncio
+import contextlib
 import enum
 import os.path
 import time
@@ -51,7 +52,7 @@ from PyQt5.QtWidgets import (QPushButton, QLabel, QMessageBox, QHBoxLayout,
                              QFileDialog, QWidget, QToolButton, QTreeView, QPlainTextEdit,
                              QHeaderView, QApplication, QToolTip, QTreeWidget, QStyledItemDelegate,
                              QMenu, QStyleOptionViewItem, QLayout, QLayoutItem, QAbstractButton,
-                             QGraphicsEffect, QGraphicsScene, QGraphicsPixmapItem, QSizePolicy)
+                             QGraphicsEffect, QGraphicsScene, QGraphicsPixmapItem, QSizePolicy, QAction)
 
 from electrum.i18n import _, languages
 from electrum.util import FileImportFailed, FileExportFailed, make_aiohttp_session, resource_path
@@ -59,10 +60,14 @@ from electrum.util import EventListener, event_listener
 from electrum.invoices import PR_UNPAID, PR_PAID, PR_EXPIRED, PR_INFLIGHT, PR_UNKNOWN, PR_FAILED, PR_ROUTING, PR_UNCONFIRMED
 from electrum.logging import Logger
 from electrum.qrreader import MissingQrDetectionLib
+from electrum.simple_config import ConfigVarWithConfig
+
+from electrum.gui import messages
 
 from .util import read_QIcon
 
 if TYPE_CHECKING:
+    from electrum import SimpleConfig
     from .main_window import ElectrumWindow
 
 
@@ -73,28 +78,41 @@ class MyMenu(QMenu):
         self.setToolTipsVisible(True)
         self.config = config
 
-    def addToggle(self, text: str, callback, *, tooltip=''):
+    def addToggle(self, text: str, callback, *, tooltip='') -> QAction:
         m = self.addAction(text, callback)
         m.setCheckable(True)
         m.setToolTip(tooltip)
         return m
 
-    def addConfig(self, text:str, name:str, default:bool, *, tooltip='', callback=None):
-        b = self.config.get(name, default)
-        m = self.addAction(text, lambda: self._do_toggle_config(name, default, callback))
+    def addConfig(
+        self,
+        configvar: 'ConfigVarWithConfig',
+        *,
+        callback=None,
+        checked: Optional[bool] = None,  # to override initial state of checkbox
+        short_desc: Optional[str] = None,
+    ) -> QAction:
+        assert isinstance(configvar, ConfigVarWithConfig), configvar
+        if short_desc is None:
+            short_desc = configvar.get_short_desc()
+            assert short_desc is not None, f"short_desc missing for {configvar}"
+        if checked is None:
+            checked = bool(configvar.get())
+        m = self.addAction(short_desc, lambda: self._do_toggle_config(configvar, callback=callback))
         m.setCheckable(True)
-        m.setChecked(b)
-        m.setToolTip(tooltip)
+        m.setChecked(checked)
+        if (long_desc := configvar.get_long_desc()) is not None:
+            m.setToolTip(messages.to_rtf(long_desc))
         return m
 
-    def _do_toggle_config(self, name, default, callback):
-        b = self.config.get(name, default)
-        self.config.set_key(name, not b)
+    def _do_toggle_config(self, configvar: 'ConfigVarWithConfig', *, callback):
+        b = configvar.get()
+        configvar.set(not b)
         if callback:
             callback()
 
 
-def create_toolbar_with_menu(config, title):
+def create_toolbar_with_menu(config: 'SimpleConfig', title):
     menu = MyMenu(config)
     toolbar_button = QToolButton()
     toolbar_button.setIcon(read_QIcon("preferences.png"))
@@ -125,7 +143,7 @@ class MySortModel(QSortFilterProxyModel):
         v2 = item2.text()
         try:
             return Decimal(v1) < Decimal(v2)
-        except:
+        except Exception:
             return v1 < v2
 
 class ElectrumItemDelegate(QStyledItemDelegate):
@@ -386,7 +404,7 @@ class MyTreeView(QTreeView):
         for row in range(self.model().rowCount()):
             self.hide_row(row)
 
-    def create_toolbar(self, config):
+    def create_toolbar(self, config: 'SimpleConfig'):
         return
 
     def create_toolbar_buttons(self):
@@ -401,7 +419,15 @@ class MyTreeView(QTreeView):
     def create_toolbar_with_menu(self, title):
         return create_toolbar_with_menu(self.config, title)
 
-    def show_toolbar(self, state, config=None):
+    configvar_show_toolbar = None  # type: Optional[ConfigVarWithConfig]
+    _toolbar_checkbox = None  # type: Optional[QAction]
+    def show_toolbar(self, state: bool = None):
+        if state is None:  # get value from config
+            if self.configvar_show_toolbar:
+                state = self.configvar_show_toolbar.get()
+            else:
+                return
+        assert isinstance(state, bool), state
         if state == self.toolbar_shown:
             return
         self.toolbar_shown = state
@@ -409,9 +435,18 @@ class MyTreeView(QTreeView):
             b.setVisible(state)
         if not state:
             self.on_hide_toolbar()
+        if self._toolbar_checkbox is not None:
+            # update the cb state now, in case the checkbox was not what triggered us
+            self._toolbar_checkbox.setChecked(state)
 
-    def toggle_toolbar(self, config=None):
-        self.show_toolbar(not self.toolbar_shown, config)
+    def on_hide_toolbar(self):
+        pass
+
+    def toggle_toolbar(self):
+        new_state = not self.toolbar_shown
+        self.show_toolbar(new_state)
+        if self.configvar_show_toolbar:
+            self.configvar_show_toolbar.set(new_state)
 
     def add_copy_menu(self, menu: QMenu, idx) -> QMenu:
         cc = menu.addMenu(_("Copy"))
